# -*- coding: utf-8 -*-
"""Build and install Shadow3"""
from __future__ import division, absolute_import, print_function

try:
    from pykern import pksetup
except ImportError:
    import pip
    pip.main(['install', 'pykern'])
    from pykern import pksetup

# Must be imported AFTER pykern import, because numpy monkey patches setuptools
import numpy.distutils.core
import numpy.distutils.fcompiler
import platform


class FCompiler(numpy.distutils.fcompiler.FCompiler, object):
    """Remove -Wall and -fno-second-underscore flags from cmd

    The flags -fno-second-underscore and -Wall are hardwired in
    various numpy classes so we have to remove them dynamically,
    because Shadow won't compile with them.

    There doesn't seem to be a method to edit these flags.
    """
    def spawn(self, cmd, *args, **kwargs):
        for f in ('-Wall', '-fno-second-underscore'):
            if f in cmd:
                cmd.remove(f)
        super(FCompiler, self).spawn(cmd, *args, **kwargs)

# Monkey patch FCompiler class
numpy.distutils.fcompiler.FCompiler = FCompiler

extension_args = {}
p = platform.system()
if p == 'Darwin':
    macro = "_COMPILE4MAX"
    import os.path
    import subprocess
    l = subprocess.check_output(['gfortran' , '--print-file-name' , 'libgfortran.dylib'])
    extension_args = {
        'library_dirs': os.path.dirname(l),
        'extra_link_args': ['-Wl,-no_compact_unwind'],
    }
elif p == 'Linux':
    macro = "_COMPILE4NIX"
elif p == 'Windows':
    macro = "_COMPILE4WIN"
else:
    raise AssertionError('{}: unhandled platform'.format(p))


pksetup.setup(
    name='shadow3',
    packages=['Shadow'],
    url='http://github.com/srio/shadow3',
    license='http://www.gnu.org/licenses/gpl-3.0.html',
    author='Franco Cerrina and M. Sanchez del Rio',
    author_email='srio@esrf.eu',
    description='SHADOW is an open source ray tracing code for modeling optical systems.',
    ext_modules=[
        numpy.distutils.core.Extension(
            name='Shadow.ShadowLib',
            sources=['src/c/shadow_bind_python.c'],
            include_dirs=['src/c', 'src/def', numpy.get_include()],
            libraries=['shadow3c', 'gfortran'],
            **extension_args
        ),
    ],
    pksetup={
        'extra_directories': ['src', 'examples', 'manual-tests'],
        'numpy_distutils': True,
        'pip_install': False,
    },
    libraries=[
        ('shadow3c', {
            'some-random-param': 1,
            'sources': [
                'src/c/shadow_bind_c.c',
                # The order of these files matters, because fortran
                # compilers need the "*.mod" files for "use" statements
                # 'fortran/shadow_version.f90',
                'src/fortran/shadow_globaldefinitions.f90',
                'src/fortran/stringio.f90',
                'src/fortran/gfile.f90',
                'src/fortran/shadow_beamio.f90',
                'src/fortran/shadow_math.f90',
                'src/fortran/shadow_variables.f90',
                'src/fortran/shadow_roughness.f90',
                'src/fortran/shadow_kernel.f90',
                'src/fortran/shadow_synchrotron.f90',
                'src/fortran/shadow_pre_sync.f90',
                'src/fortran/shadow_pre_sync_urgent.f90',
                'src/fortran/shadow_preprocessors.f90',
                'src/fortran/shadow_postprocessors.f90',
                'src/fortran/shadow_bind_f.f90',
                'src/fortran/shadow_crl.f90',
            ],
            'macros': [(macro, 1)],
            'include_dirs': ['src/def', 'src/fortran', 'src/c'],
            'extra_f90_compile_args': ['-cpp', '-ffree-line-length-none', '-fomit-frame-pointer'],
        }),
    ],
)
